#include "Queue.hh"
#include <string>
#include <iostream>

Queue::Queue(int capacity) :
  elements {new int[capacity]},
  headIndex {0},
  tailIndex {0},
  size {0},
  capacity {capacity}
{ }

void Queue::enqueue(int item) {
  elements[tailIndex] = item;
  tailIndex = (tailIndex + 1) % capacity;
  size++;
}

int Queue::dequeue(void) {
  int h = elements[headIndex];
  headIndex = (headIndex + 1) % capacity;
  size--;
  return h;
}

int Queue::head(void) const {
  return elements[headIndex];
}

bool Queue::is_empty(void) const {
  return (headIndex == tailIndex);
}

int Queue::inspect(int i) const {
  return elements[(headIndex + i) % capacity];
}

std::string Queue::to_string(void) const {
  if (!(*this)) {
    return "[]";
  } else {
    std::string s = "[(";
    s += std::to_string((*this)());
    s += ")";
    for (int i = 1; i < size; i++) {
      s += " " + std::to_string((*this)[i]);
    }
    return s + "]";
  }
}

Queue::~Queue(void) {
  delete [] elements;
}

void Queue::operator+=(int item) {
  enqueue(item);
}

int Queue::operator--(void) {
  return dequeue();
}

bool Queue::operator!(void) const {
  return is_empty();
}

int Queue::operator()(void) const {
  return head();
}

int Queue::operator[](int i) const {
  return inspect(i);
}

std::ostream& operator<<(std::ostream& os, const Queue& q) {
  os << q.to_string();
  return os;
}

