#ifndef _COBRA_EMIT_H
#define _COBRA_EMIT_H

class Component;

class INST {
public:
  virtual void output(std::ostream& os, const Component& c) const = 0;
  virtual void place(Component& c) { }
};

class SET : public INST {
public:
  std::string dst;
  int         val;
  SET(std::string d, int v) : dst {d}, val {v} { }
  void output(std::ostream& os, const Component& c) const;
  void place(Component& c);
};

class MOV : public INST {
public:
  std::string dst;
  std::string src;
  MOV(std::string d, std::string s) : dst {d}, src {s} {}
  void output(std::ostream& os, const Component& c) const;
  void place(Component& c);
};

class RTV : public INST {
public:
  std::string dst;
  RTV(std::string d) : dst {d} {}
  void output(std::ostream& os, const Component& c) const;
  void place(Component& c);
};

class GET : public INST {
public:
  std::string dst;
  GET(std::string d) : dst {d} {} 
  void output(std::ostream& os, const Component& c) const;
  void place(Component& c);
};

class RND : public INST {
public:
  std::string dst;
  RND(std::string d) : dst {d} {} 
  void output(std::ostream& os, const Component& c) const;
  void place(Component& c);
};

class NOP : public INST {
public:
  NOP(void) { } 
  void output(std::ostream& os, const Component& c) const;
};

class PUTI : public INST {
public:
  std::string src;
  PUTI(std::string s) : src {s} { } 
  void output(std::ostream& os, const Component& c) const;
  void place(Component& c);
};

class PUTS : public INST {
public:
  std::string lbl;
  PUTS(std::string l) : lbl {l} { } 
  void output(std::ostream& os, const Component& c) const;
};

class ADD : public INST {
public:
  std::string dst;
  std::string src1;
  std::string src2;
  ADD(std::string d, std::string s1, std::string s2) :
    dst {d}, src1 {s1}, src2 {s2} {}
  void output(std::ostream& os, const Component& c) const;
  void place(Component& c);
};

class ADDI : public INST {
public:
  std::string dst;
  std::string src;
  int         val;
  ADDI(std::string d, std::string s, int v) :
    dst {d}, src {s}, val {v} {}
  void output(std::ostream& os, const Component& c) const;
  void place(Component& c);
};

class SUB : public INST {
public:
  std::string dst;
  std::string src1;
  std::string src2;
  SUB(std::string d, std::string s1, std::string s2) :
    src1 {s1}, src2 {s2}, dst {d} {}
  void output(std::ostream& os, const Component& c) const;
  void place(Component& c);
};

class MUL : public INST {
public:
  std::string dst;
  std::string src1;
  std::string src2;
  MUL(std::string d, std::string s1, std::string s2) :
    src1 {s1}, src2 {s2}, dst {d} {}
  void output(std::ostream& os, const Component& c) const;
  void place(Component& c);
};

class RTN : public INST {
public:
  std::string src;
  RTN(std::string s) : src {s} {}
  void output(std::ostream& os, const Component& c) const;
  void place(Component& c);
};

class BNE : public INST {
public:
  std::string src1;
  std::string src2;
  std::string lbl;
  BNE(std::string  s1, std::string s2, std::string l) :
    src1 {s1}, src2 {s2}, lbl {l} {}
  void output(std::ostream& os, const Component& c) const;
  void place(Component& c);
};

class BGE : public INST {
public:
  std::string src1;
  std::string src2;
  std::string lbl;
  BGE(std::string  s1, std::string s2, std::string l) :
    src1 {s1}, src2 {s2}, lbl {l} {}
  void output(std::ostream& os, const Component& c) const;
  void place(Component& c);
};

class BRA : public INST {
public:
  std::string lbl;
  BRA(std::string l) : lbl {l} {}
  void output(std::ostream& os, const Component& c) const;
};

class CLL : public INST {
public:
  std::string lbl;
  CLL(std::string l) : lbl {l} {}
  void output(std::ostream& os, const Component& c) const;
};

class LBL : public INST {
public:
  std::string lbl;
  LBL(std::string l) : lbl {l} {}
  void output(std::ostream& os, const Component& c) const;
};

class ARG : public INST {
public:
  int idx;
  std::string src;
  ARG(int i, std::string s) : idx {i}, src {s} {}
  void output(std::ostream& os, const Component& c) const;
  void place(Component& c);
};

class ENTER : public INST {
public:
  ENTER(void) {}
  void output(std::ostream& os, const Component& c) const;
};

class LEAVE : public INST {
public:
  LEAVE(void) {}
  void output(std::ostream& os, const Component& c) const;
};

#endif
