%{ /* -*- C++ -*- */
  #include <string>
  #include "cobra.hh"
  #include "cobra-parser.hh"
  static yy::location loc;
%}

%option noyywrap
%option nounput batch debug noinput
   
nmbr   0|[1-9][0-9]*
name   [_a-zA-Z][_a-zA-Z0-9]*
strg  \"(\\.|[^\"\\])*\"
blnk  [ \t]

%{
  // Code run each time a pattern is matched.
  #define YY_USER_ACTION  loc.columns (yyleng);
%}

%%

%{
  // Code run each time yylex is called.
  loc.step();
%}

{blnk}+   loc.step();
[\n]+      loc.lines(yyleng); loc.step();
"+"        return yy::cobra_parser::make_PLUS(loc);
"-"        return yy::cobra_parser::make_MINUS(loc);
"*"        return yy::cobra_parser::make_TIMES(loc);
":"        return yy::cobra_parser::make_COLON(loc);
","        return yy::cobra_parser::make_COMMA(loc);
"+="       return yy::cobra_parser::make_UPDATE(loc);
"=="       return yy::cobra_parser::make_EQUALS(loc);
"="        return yy::cobra_parser::make_ASSIGN(loc);
"<"        return yy::cobra_parser::make_LESS(loc);
"<="       return yy::cobra_parser::make_LEQUAL(loc);
"not"      return yy::cobra_parser::make_NOT(loc);
"and"      return yy::cobra_parser::make_AND(loc);
"or"       return yy::cobra_parser::make_OR(loc);
"("        return yy::cobra_parser::make_LPAREN(loc);
")"        return yy::cobra_parser::make_RPAREN(loc);
"def"      return yy::cobra_parser::make_DEF(loc);
"begin"    return yy::cobra_parser::make_BEGIN(loc);
"end"      return yy::cobra_parser::make_END(loc);
"for"      return yy::cobra_parser::make_FOR(loc);
"to"       return yy::cobra_parser::make_TO(loc);
"while"    return yy::cobra_parser::make_WHILE(loc);
"if"       return yy::cobra_parser::make_IF(loc);
"else"     return yy::cobra_parser::make_ELSE(loc);
"print"    return yy::cobra_parser::make_PRINT(loc);
"input"    return yy::cobra_parser::make_INPUT(loc);
"random"   return yy::cobra_parser::make_RANDOM(loc);
"return"   return yy::cobra_parser::make_RETURN(loc);
"pass"     return yy::cobra_parser::make_PASS(loc);
{nmbr}     {
             errno = 0;
             int vlu = std::stoi(yytext);
             return yy::cobra_parser::make_NMBR(vlu,loc);
           }
{name}     {
             errno = 0;
             std::string nme = yytext;
             return yy::cobra_parser::make_NAME(nme,loc);
           }
{strg}     {
             errno = 0;
             std::string str = yytext;
             return yy::cobra_parser::make_STRG(str.substr(1,str.length()-2),loc);
           }
.          {
             std::string s = yytext;
             driver.error(loc, "Invalid text scanned '"+s+"'.");
	   }
<<EOF>>    return yy::cobra_parser::make_EOF(loc);
%%

void cobra::scan_begin(void) {
  yy_flex_debug = trace_scanning;
  if (file.empty() || file == "-") {
    yyin = stdin;
  } else if (!(yyin = fopen(file.c_str(), "r"))) {
    error ("cannot open " + file + ": " + strerror(errno));
    exit (EXIT_FAILURE);
  }
}

void cobra::scan_end(void) {
  fclose (yyin);
}
