#include <memory>
#include "Arithmetic.hh"

Plus::Plus(std::shared_ptr<Expression> l,std::shared_ptr<Expression> r) :
  left {l}, right {r} { }

Times::Times(std::shared_ptr<Expression> l,std::shared_ptr<Expression> r) :
  left {l}, right {r} { }

Negation::Negation(std::shared_ptr<Expression> e) :
  expn {e} { }

Number::Number(int n) :
  value {n} { }

LookUp::LookUp(std::string x) :
  name {x} { }

int Plus::evaluate(std::unordered_map<std::string,int> variables) const {
  return left->evaluate(variables) + right->evaluate(variables);
}

int Times::evaluate(std::unordered_map<std::string,int> variables) const {
  return left->evaluate(variables) * right->evaluate(variables);
}

int Negation::evaluate(std::unordered_map<std::string,int> variables) const {
  return - expn->evaluate(variables);
}

int Number::evaluate(std::unordered_map<std::string,int> variables) const {
  return value;
}

int LookUp::evaluate(std::unordered_map<std::string,int> variables) const {
  return variables[name];
}

std::string Plus::to_string(void) const {
  return "(" + left->to_string() + " + " + right->to_string() + ")";
}

std::string Times::to_string(void) const {
  return "(" + left->to_string() + " * " + right->to_string() + ")";
}

std::string Negation::to_string(void) const {
  return "(-" + expn->to_string() + ")";
}

std::string Number::to_string(void) const {
  return std::to_string(value);
}

std::string LookUp::to_string(void) const {
  return name;
}
