#ifndef _ARITHMETIC_H
#define _ARITHMETIC_H

#include <string>
#include <memory>
#include <unordered_map>

class Expression {
public:
  virtual int evaluate(std::unordered_map<std::string,int> variables) const = 0;
  virtual std::string to_string(void) const = 0;
};

class Plus : public Expression {
private:
  std::shared_ptr<Expression> left;
  std::shared_ptr<Expression> right;
public:
  Plus(std::shared_ptr<Expression> l,std::shared_ptr<Expression> r);
  int evaluate(std::unordered_map<std::string,int> variables) const;
  std::string to_string(void) const;
};

class Times : public Expression {
private:
  std::shared_ptr<Expression> left;
  std::shared_ptr<Expression> right;
public:
  Times(std::shared_ptr<Expression> l,std::shared_ptr<Expression> r);
  int evaluate(std::unordered_map<std::string,int> variables) const;
  std::string to_string(void) const;
};

class Negation : public Expression {
private:
  std::shared_ptr<Expression> expn;
public:
  Negation(std::shared_ptr<Expression> e);
  int evaluate(std::unordered_map<std::string,int> variables) const;
  std::string to_string(void) const;
};

class Number : public Expression {
private:
  int value;
public:
  Number(int n);
  int evaluate(std::unordered_map<std::string,int> variables) const;
  std::string to_string(void) const;
};

class LookUp : public Expression {
private:
  std::string name;
public:
  LookUp(std::string x);
  int evaluate (std::unordered_map<std::string,int> variables) const;
  std::string to_string(void) const;
};

#endif
